@@ -1,20 +1,11 @@
-from djangoldp.permissions import LDPPermissions
-from django.db.models.base import ModelBase
 from django.conf import settings
+from django.db.models import QuerySet, Q
+from django.db.models.base import ModelBase
 
-XMPP_SERVERS = set({'51.15.243.248', '212.47.234.179', '2001:bc8:47b0:2711::1'})
+from djangoldp.permissions import LDPPermissions
+from djangoldp_circle.filters import CircleFilterBackend, CircleMemberFilterBackend
+from djangoldp_circle.xmpp import get_client_ip, XMPP_SERVERS
 
-if hasattr(settings, 'XMPP_SERVER_IP'):
-    XMPP_SERVERS = XMPP_SERVERS.union(getattr(settings, 'XMPP_SERVER_IP'))
-
-
-def get_client_ip(request):
-    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
-    if x_forwarded_for:
-        ip = x_forwarded_for.split(',')[0]
-    else:
-        ip = request.META.get('REMOTE_ADDR')
-    return ip
 
 def _append_circle_object_permissions(perms, obj, user):
     '''Auxiliary function analyses a circle object for member permissions'''
@@ -31,7 +22,10 @@ def _append_circle_object_permissions(perms, obj, user):
 
     return perms
 
+
 class CirclePermissions(LDPPermissions):
+    filter_backends = [CircleFilterBackend]
+
     def user_permissions(self, user, obj_or_model, obj=None):
         if not isinstance(obj_or_model, ModelBase):
             obj = obj_or_model
@@ -41,7 +35,7 @@ class CirclePermissions(LDPPermissions):
 
         if not user.is_anonymous:
             # object-level permissions
-            if obj:
+            if obj and not isinstance(obj, ModelBase):
                 perms = _append_circle_object_permissions(perms, obj, user)
 
             # model-level permissions
@@ -64,6 +58,8 @@ class CirclePermissions(LDPPermissions):
 
 
 class CircleMemberPermissions(LDPPermissions):
+    filter_backends = [CircleMemberFilterBackend]
+
     def user_permissions(self, user, obj_or_model, obj=None):
         if not isinstance(obj_or_model, ModelBase):
             obj = obj_or_model
@@ -73,7 +69,7 @@ class CircleMemberPermissions(LDPPermissions):
 
         if not user.is_anonymous:
             # object-level permissions
-            if obj and hasattr(obj, 'user'):
+            if obj and hasattr(obj, 'user') and not isinstance(obj, ModelBase):
                 # the operation is on myself
                 if obj.user == user:
                     perms.add('view')
@@ -95,7 +91,7 @@ class CircleMemberPermissions(LDPPermissions):
                         perms = perms.union({'view', 'add'})
 
                         # permissions gained for admins (on other users)
-                        if obj.circle.members.filter(user=user).get().is_admin\
+                        if obj.circle.members.filter(user=user).get().is_admin \
                                 and not obj.is_admin:
                             perms = perms.union({'delete', 'change'})
 
